/*
 * Copyright 2008 Pentaho Corporation.  All rights reserved. 
 * This software was developed by Pentaho Corporation and is provided under the terms 
 * of the Mozilla Public License, Version 1.1, or any later version. You may not use 
 * this file except in compliance with the license. If you need a copy of the license, 
 * please go to http://www.mozilla.org/MPL/MPL-1.1.txt. The Original Code is the Pentaho 
 * BI Platform.  The Initial Developer is Pentaho Corporation.
 *
 * Software distributed under the Mozilla Public License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or  implied. Please refer to 
 * the license for the specific language governing your rights and limitations.
 */
package org.pentaho.iphone;

/**
 * This Class overrides the setupRequestHandler method in Pentaho's ViewAction
 * Servlet, allowing for iphone support.  
 * 
 * Technical Note: Due to a similar call in ComponentBase, the default-parameter-xsl
 * property specified in pentaho.xml should be removed.
 * 
 * @author Will Gorman (wgorman@pentaho.com)
 */
import javax.servlet.http.HttpServletRequest;

import org.pentaho.platform.api.engine.IParameterProvider;
import org.pentaho.platform.api.engine.IPentahoSession;
import org.pentaho.platform.api.engine.ISystemSettings;
import org.pentaho.platform.engine.core.system.PentahoSystem;
import org.pentaho.platform.web.servlet.HttpServletRequestHandler;
import org.pentaho.platform.web.servlet.ViewAction;

public class PentahoiPhoneViewAction extends ViewAction {

  private static final long serialVersionUID = -6112753523568505826L;

  @Override
  protected void setupRequestHandler(HttpServletRequest request, HttpServletRequestHandler requestHandler,
      IParameterProvider requestParameters, IPentahoSession userSession) {
    this.setupRequestHandler(request, requestHandler);
    // first take a secondary action
    String actionName = requestParameters.getStringParameter("action2", null); //$NON-NLS-1$
    if (actionName == null) {
      // now look for a primary action
      actionName = requestParameters.getStringParameter("action", null); //$NON-NLS-1$
    }
    
    ISystemSettings systemSettings = PentahoSystem.getSystemSettings();
    String defaultParameterXsl = systemSettings.getSystemSetting("default-parameter-xsl", "DefaultParameterForm.xsl"); //$NON-NLS-1$ //$NON-NLS-2$
    
    //
    // Added for iPhone override support
    //
    
    if (request.getAttribute("default-parameter-xsl") != null) {  //$NON-NLS-1$ //$NON-NLS-2$
      defaultParameterXsl = (String)request.getAttribute("default-parameter-xsl");
    }
    System.out.println("DEFAULT PARAM XSL: " + defaultParameterXsl);   
    requestHandler.setParameterXsl(defaultParameterXsl);
    if (doSubscribe(requestParameters)) {
      requestHandler.setForcePrompt(true);
      requestHandler.setParameterProvider("PRO_EDIT_SUBSCRIPTION", requestParameters); //$NON-NLS-1$ 
    } 
  }
}
